searchData={"items":[{"type":"module","title":"ConCache","doc":"Implements an ETS based key/value storage with following additional features:\n\n- row level synchronized writes (inserts, read/modify/write updates, deletes)\n- TTL support\n- modification callbacks\n\nExample usage:\n\n    ConCache.start_link(name: :my_cache, ttl_check_interval: false)\n    ConCache.put(:my_cache, :foo, 1)\n    ConCache.get(:my_cache, :foo)  # 1\n\nThe following rules apply:\n\n- Modifications are by isolated per row. Two processes can't modify the same\n  row at the same time. Dirty operations are available through `dirty_` equivalents.\n- Reads are dirty by default. You can use `isolated/4` to perform isolated custom\n  operations.\n- Operations are always performed in the caller process. Custom lock implementation\n  is used to ensure synchronism. See `README.md` for more details.\n- In this example, items don't expire. See `start_link/1` for details on how to setup expiry.\n\nSee `start_link/1` for more details.","ref":"ConCache.html"},{"type":"function","title":"ConCache.delete/2","doc":"Deletes the item from the cache.","ref":"ConCache.html#delete/2"},{"type":"function","title":"ConCache.dirty_delete/2","doc":"Dirty equivalent of `delete/2`.","ref":"ConCache.html#dirty_delete/2"},{"type":"function","title":"ConCache.dirty_fetch_or_store/3","doc":"Dirty equivalent of `fetch_or_store/3`.","ref":"ConCache.html#dirty_fetch_or_store/3"},{"type":"function","title":"ConCache.dirty_get_or_store/3","doc":"Dirty equivalent of `get_or_store/3`.","ref":"ConCache.html#dirty_get_or_store/3"},{"type":"function","title":"ConCache.dirty_insert_new/3","doc":"Dirty equivalent of `insert_new/3`.","ref":"ConCache.html#dirty_insert_new/3"},{"type":"function","title":"ConCache.dirty_put/3","doc":"Dirty equivalent of `put/3`.","ref":"ConCache.html#dirty_put/3"},{"type":"function","title":"ConCache.dirty_update/3","doc":"Dirty equivalent of `update/3`.","ref":"ConCache.html#dirty_update/3"},{"type":"function","title":"ConCache.dirty_update_existing/3","doc":"Dirty equivalent of `update_existing/3`.","ref":"ConCache.html#dirty_update_existing/3"},{"type":"function","title":"ConCache.ets/1","doc":"Returns the ets table managed by the cache.","ref":"ConCache.html#ets/1"},{"type":"function","title":"ConCache.fetch_or_store/3","doc":"Retrieves the item from the cache, or inserts the new item.\n\nIf the item exists in the cache, it is retrieved. Otherwise, the lambda\nfunction is executed and its result is stored under the given key, but only if\nit returns an `{:ok, value}` tuple. If the `{:error, reason}` tuple is returned,\ncaching is not done and the error becomes the result of the function. If the lambda\nreturns none of the above, a `RuntimeError` is raised.\n\nThe lambda may return either a plain value or `%ConCache.Item{}`.\n\nThis function is not supported by `:bag` and `:duplicate_bag` ETS tables.\n\nNote: if the item is already in the cache, this function amounts to a simple get\nwithout any locking, so you can expect it to be fairly fast.\n\nEmits `[:con_cache, :stats, :hit]` telemetry event if the item is found, \nand `[:con_cache, :stats, :miss]` otherwise.","ref":"ConCache.html#fetch_or_store/3"},{"type":"function","title":"ConCache.get/2","doc":"Reads the item from the cache.\n\nA read is always \"dirty\", meaning it doesn't block while someone is updating\nthe item under the same key. A read doesn't expire TTL of the item, unless\n`touch_on_read` option is set while starting the cache.\n\nEmits `[:con_cache, :stats, :hit]` telemetry event if the item is found, \nand `[:con_cache, :stats, :miss]` otherwise.","ref":"ConCache.html#get/2"},{"type":"function","title":"ConCache.get_or_store/3","doc":"Retrieves the item from the cache, or inserts the new item.\n\nIf the item exists in the cache, it is retrieved. Otherwise, the lambda\nfunction is executed and its result is stored under the given key.\n\nThe lambda may return either a plain value or `%ConCache.Item{}`.\n\nThis function is not supported by `:bag` and `:duplicate_bag` ETS tables.\n\nNote: if the item is already in the cache, this function amounts to a simple get\nwithout any locking, so you can expect it to be fairly fast.\n\nEmits `[:con_cache, :stats, :hit]` telemetry event if the item is found, \nand `[:con_cache, :stats, :miss]` otherwise.","ref":"ConCache.html#get_or_store/3"},{"type":"function","title":"ConCache.insert_new/3","doc":"Inserts the item into the cache unless it exists.","ref":"ConCache.html#insert_new/3"},{"type":"function","title":"ConCache.isolated/4","doc":"Isolated execution over arbitrary lock in the cache.\n\nYou can do whatever you want in the function, not necessarily related to the\ncache. The return value is the result of the provided lambda.\n\nThis allows you to perform flexible isolation. If you use the key\nof your item as a `key`, then this operation will be exclusive to\nupdates. This can be used e.g. to perform isolated reads:\n\n    # Process A:\n    ConCache.isolated(:my_cache, :my_item_key, fn() -> ... end)\n\n    # Process B:\n    ConCache.update(:my_cache, :my_item, fn(old_value) -> ... end)\n\nThese two operations are mutually exclusive.","ref":"ConCache.html#isolated/4"},{"type":"function","title":"ConCache.put/3","doc":"Stores the item into the cache.","ref":"ConCache.html#put/3"},{"type":"function","title":"ConCache.size/1","doc":"Returns the number of items stored in the cache.","ref":"ConCache.html#size/1"},{"type":"function","title":"ConCache.start_link/1","doc":"Starts the server and creates an ETS table.\n\nOptions:\n  - `{:name, atom} - A name of the cache process.`\n  - `{:ttl_check_interval, time_ms | false}` - Required. A check interval for TTL expiry.\n    Provide a positive integer for expiry to work, or pass `false` to disable ttl checks.\n    See below for more details on expiry.\n  - `{:global_ttl, time_ms | :infinity}` - The time after which an item expires.\n    When an item expires, it is removed from the cache. Updating the item\n    extends its expiry time.\n  - `{:touch_on_read, true | false}` - Controls whether read operation extends\n    expiry of items. False by default.\n  - `{:callback, callback_fun}` - If provided, this function is invoked __after__\n    an item is inserted or updated, or __before__ it is deleted.\n  - `{:acquire_lock_timeout, timeout_ms}` - The time a client process waits for\n    the lock. Default is 5000.\n  - `{:ets_options, [ets_option]` – The options for ETS process.\n  - `{:n_lock_partitions, pos_integer}` - A positive integer representing\n    the desired number of lock partitions\n\nIn addition, following ETS options are supported:\n  - `:set` - An ETS table will be of the `:set` type (default).\n  - `:ordered_set` - An ETS table will be of the `:ordered_set` type.\n  - `:bag` - An ETS table will be of the `:bag` type.\n  - `:duplicate_bag` - An ETS table will be of the `:duplicate_bag` type.\n  - `:named_table`\n  - `:name`\n  - `:heir`\n  - `:write_concurrency`\n  - `:read_concurrency`\n  - `:decentralized_counters`","ref":"ConCache.html#start_link/1"},{"type":"function","title":"Child specification - ConCache.start_link/1","doc":"To insert your cache into the supervision tree, pass the child specification\nin the shape of `{ConCache, con_cache_options}`. For example:\n\n```\n{ConCache, [name: :my_cache, ttl_check_interval: false]}\n```","ref":"ConCache.html#start_link/1-child-specification"},{"type":"function","title":"Expiry - ConCache.start_link/1","doc":"To configure expiry, you need to provide positive integer for the\n`:ttl_check_interval` option. This integer represents the millisecond interval\nin which the expiry is performed. You also need to provide the `:global_ttl`\noption, which represents the default TTL time for the item.\n\nTTL of each item is by default extended only on modifications. This can be\nchanged with the `touch_on_read: true` option.\n\nIf you need a granular control of expiry per each item, you can pass a\n`ConCache.Item` struct when storing data.\n\nIf you don't want a modification of an item to extend its TTL, you can pass a\n`ConCache.Item` struct, with `:ttl` field set to `:no_update`.","ref":"ConCache.html#start_link/1-expiry"},{"type":"function","title":"Choosing ttl_check_interval time - ConCache.start_link/1","doc":"When expiry is configured, the owner process works in discrete steps, doing\ncleanups every `ttl_check_interval` milliseconds. This approach allows the owner\nprocess to do fairly small amount of work in each discrete step.\n\nAssuming there's no huge system overload, an item's max lifetime is thus\n`global_ttl + ttl_check_interval` [ms], after the last item's update.\n\nThus, a lower value of ttl_check_interval time means more frequent purging which may\nreduce your memory consumption, but could also cause performance penalties.\nHigher values put less pressure on processing, but item expiry is less precise.","ref":"ConCache.html#start_link/1-choosing-ttl_check_interval-time"},{"type":"function","title":"ConCache.touch/2","doc":"Manually touches the item to prolongate its expiry.","ref":"ConCache.html#touch/2"},{"type":"function","title":"ConCache.try_isolated/4","doc":"Similar to `isolated/4` except it doesn't wait for the lock to be available.\n\nIf the lock can be acquired immediately, it will be acquired and the function\nwill be invoked. Otherwise, an error is returned immediately.","ref":"ConCache.html#try_isolated/4"},{"type":"function","title":"ConCache.update/3","doc":"Updates the item, or stores new item if it doesn't exist.\n\nThe `update_fun` is invoked after the item is locked. Here, you can be certain\nthat no other process will update this item, unless they are doing dirty updates\nor writing directly to the underlying ETS table. This function is not supported\nby `:bag` or `:duplicate_bag` ETS tables.\n\nThe updater lambda must return one of the following:\n\n  - `{:ok, value}` - causes the value to be stored into the table\n  - `{:error, reason}` - the value won't be stored and `{:error, reason}` will be returned","ref":"ConCache.html#update/3"},{"type":"function","title":"ConCache.update_existing/3","doc":"Updates the item only if it exists. Otherwise works just like `update/3`.","ref":"ConCache.html#update_existing/3"},{"type":"type","title":"ConCache.callback_fun/0","doc":"","ref":"ConCache.html#t:callback_fun/0"},{"type":"type","title":"ConCache.ets_option/0","doc":"","ref":"ConCache.html#t:ets_option/0"},{"type":"type","title":"ConCache.fetch_or_store_fun/0","doc":"","ref":"ConCache.html#t:fetch_or_store_fun/0"},{"type":"type","title":"ConCache.key/0","doc":"","ref":"ConCache.html#t:key/0"},{"type":"type","title":"ConCache.options/0","doc":"","ref":"ConCache.html#t:options/0"},{"type":"type","title":"ConCache.store_fun/0","doc":"","ref":"ConCache.html#t:store_fun/0"},{"type":"type","title":"ConCache.store_value/0","doc":"","ref":"ConCache.html#t:store_value/0"},{"type":"type","title":"ConCache.t/0","doc":"","ref":"ConCache.html#t:t/0"},{"type":"type","title":"ConCache.update_fun/0","doc":"","ref":"ConCache.html#t:update_fun/0"},{"type":"type","title":"ConCache.value/0","doc":"","ref":"ConCache.html#t:value/0"},{"type":"module","title":"ConCache.Item","doc":"This struct can be used in place of naked values to set per-item TTL values.","ref":"ConCache.Item.html"},{"type":"type","title":"ConCache.Item.t/0","doc":"","ref":"ConCache.Item.html#t:t/0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"1.1.1 - Changelog","doc":"- slight performance improvement","ref":"changelog.html#1-1-1"},{"type":"extras","title":"1.1 - Changelog","doc":"- basic telemetry support","ref":"changelog.html#1-1"},{"type":"extras","title":"1.0 - Changelog","doc":"- no new changes from 0.14.0","ref":"changelog.html#1-0"},{"type":"extras","title":"0.14.0 - Changelog","doc":"- Requires Elixir 1.7 or newer\n- Added `fetch_or_store/3` and `dirty_fetch_or_store/3`","ref":"changelog.html#0-14-0"},{"type":"extras","title":"0.13.1 - Changelog","doc":"- removed a few compiler warnings","ref":"changelog.html#0-13-1"},{"type":"extras","title":"0.13.0 - Changelog","doc":"","ref":"changelog.html#0-13-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Requires Elixir 1.5 or newer\n- The `ConCache.start_link` function takes only one argument. Previously, you needed to pass two keyword lists, which have now been unified in a single kw list. See `ConCache.start_link/1` for details.\n- The `ConCache.start_link` parameters `:ttl_check` and `:ttl` are renamed to `:ttl_check_interval` and `:global_ttl`.\n- The `:ttl_check_interval` parameter is now required. If you don't want expiry in your cache, you need to explicitly pass `ttl_check_interval: false`.\n- If the `:ttl_check_interval` option is set to a positive integer, you also need to pass the `:global_ttl` option.\n- If a cache is configured for expiry, but you want some item to not expire, you need to pass the atom `:infinity` as its TTL value (previously, it was 0).","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements - Changelog","doc":"- Added `child_spec/1`. A `ConCache` child can now be specified as `{ConCache, [name: :my_cache, ttl_check_interval: false]}`.","ref":"changelog.html#improvements"},{"type":"extras","title":"0.12.1 - Changelog","doc":"- Relaxed version requirement for Elixir\n- Proper early exit when the cache doesn't exist","ref":"changelog.html#0-12-1"},{"type":"extras","title":"0.12.0 - Changelog","doc":"","ref":"changelog.html#0-12-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Elixir 1.4 is now required.\n- The process started through `ConCache.start_link` is a supervisor (previously it was a worker). Make sure to adapt your supervisor specifications accordingly.\n- `ConCache.start` has been removed.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improvements - Changelog","doc":"- You can now use `bag`, and `duplicate_bag` (thanks to [fcevado](https://github.com/fcevado) for implementing it).\n- Lock processes are now specific for each cache instance (previously they were shared between all of them). Multiple cache instances in the same system will not block each other.","ref":"changelog.html#improvements"},{"type":"extras","title":"0.11.1 - Changelog","doc":"- Fix warnings on 1.3.0","ref":"changelog.html#0-11-1"},{"type":"extras","title":"0.11.0 - Changelog","doc":"","ref":"changelog.html#0-11-0"},{"type":"extras","title":"Improvements - Changelog","doc":"- Support the avoiding prolongation of ttls when updated items through the `:no_update` ttl value in `%ConCache.Item{}`","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"- New items inserted with `ConCache.update/3` and `ConCache.dirty_update/3` never expired.","ref":"changelog.html#fixes"},{"type":"extras","title":"0.10.0 - Changelog","doc":"","ref":"changelog.html#0-10-0"},{"type":"extras","title":"Improvements - Changelog","doc":"- add `ConCache.size/1`","ref":"changelog.html#improvements"},{"type":"extras","title":"0.9.0 - Changelog","doc":"","ref":"changelog.html#0-9-0"},{"type":"extras","title":"Fixes - Changelog","doc":"- Support for Elixir 1.1","ref":"changelog.html#fixes"},{"type":"extras","title":"0.8.1 - Changelog","doc":"","ref":"changelog.html#0-8-1"},{"type":"extras","title":"Fixes - Changelog","doc":"- Proper unlocking of an item. Previously it was possible that a process keeps the resource locked forever if the lock attempt timed out.","ref":"changelog.html#fixes"},{"type":"extras","title":"0.8.0 - Changelog","doc":"","ref":"changelog.html#0-8-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Removed following `ConCache` functions: `size/1`, `memory/1`, `memory_bytes/1`, `get_all/1`, `with_existing/3`\n- Changed `ConCache` update functions: `update/3`, `dirty_update/3`, `update_existing/3` and `dirty_update_existing/3`. The provided lambda now must return either `{:ok, new_value}` or `{:error, reason}`.\n- Changed `ConCache.try_isolated/4` - the function returns `{:ok, result}` or `{:error, reason}`\n- Upgraded to the most recent ExActor","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fixed possible race-conditions on client process crash\n- Fixed mutual exclusion of independent caches","ref":"changelog.html#fixes"},{"type":"extras","title":"0.6.1 - Changelog","doc":"- Elixir v1.0.0","ref":"changelog.html#0-6-1"},{"type":"extras","title":"0.5.1 - Changelog","doc":"- bugfix: balanced lock wasn't working properly","ref":"changelog.html#0-5-1"},{"type":"extras","title":"0.5.0 - Changelog","doc":"- upgrade to Elixir v1.0.0-rc1","ref":"changelog.html#0-5-0"},{"type":"extras","title":"0.4.0 - Changelog","doc":"- upgrade to Elixir v0.15.0","ref":"changelog.html#0-4-0"},{"type":"extras","title":"0.3.0 - Changelog","doc":"With this version, ConCache is turned into a proper application that obeys OTP principles. This required some changes to the way ConCache is used.\n\nFirst, ConCache is now an OTP application. Consequently, you should add it as an **application dependency** in your `mix.exs`:\n\n```elixir\n  ...\n\n  def application do\n    [applications: [:con_cache, ...], ...]\n  end\n\n  ...\n```\n\nThis will make sure that before your app is started, ConCache required processes are started as well.\n\nTo create a cache, you can use `ConCache.start_link/0,1,2` or `ConCache.start/0,1,2`. These functions now return result in the form of `{:ok, pid}` where `pid` identifies the owner process of the underlying ETS table. You can use that pid as the first argument to exported functions from `ConCache` module.\n\nThe first argument to both functions must contain ConCache options (unchanged), while the second argument contains [`GenServer` start options](http://elixir-lang.org/docs/stable/elixir/GenServer.html#t:options/0). Both are by default empty lists.\n\nThe cache owner process **is not** inserted into the supervision tree of the ConCache OTP application. It is your responsibility to place it in your own tree at the desired place.\n\nOf course, when using supervision, you can't use the ETS owner process pid to interface the cache, since this process can be restarted. Instead, you must rely on some registration facility. For example, this is how you can locally register the owner process, and use the alias to interface with the cache:\n\n```elixir\n# Specifying the alias of the ETS owner process\niex(1)> ConCache.start_link([], name: :my_cache)\n\n# Interfacing the cache via the registered alias\niex(2)> ConCache.put(:my_cache, :some_key, :some_value)\niex(3)> ConCache.get(:my_cache, :some_key)\n:some_value\n```\n\nWhen creating the cache from the supervision tree, use something like this in your specification:\n\n```elixir\nworker(ConCache, [con_cache_options, [name: ...]])\n```\n\nAnd then interface the cache via the corresponding alias.\n\nBesides this simple local registration, you can also use `{:global, some_alias}`, and `{:via, module, some_alias}` format. For example, to register the process with [gproc](https://github.com/uwiger/gproc), you can do something like this:\n\n```elixir\nConCache.start_link([], name: {:via, :gproc, :my_cache})\n...\nConCache.put({:via, :gproc, :my_cache}, :some_key, :some_value)\n```","ref":"changelog.html#0-3-0"},{"type":"extras","title":"ConCache","doc":"# ConCache\n\n![Build Status](https://github.com/sasa1977/con_cache/workflows/con_cache/badge.svg?branch=master)\n[![Module Version](https://img.shields.io/hexpm/v/con_cache.svg)](https://hex.pm/packages/con_cache)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/con_cache/)\n[![Total Download](https://img.shields.io/hexpm/dt/con_cache.svg)](https://hex.pm/packages/con_cache)\n[![License](https://img.shields.io/hexpm/l/con_cache.svg)](https://github.com/sasa1977/con_cache/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/sasa1977/con_cache.svg)](https://github.com/sasa1977/con_cache/commits/master)\n\nConCache (Concurrent Cache) is an ETS based key/value storage with following additional features:\n\n- row level synchronized writes (inserts, read/modify/write updates, deletes)\n- TTL support\n- modification callbacks","ref":"readme.html"},{"type":"extras","title":"Usage in OTP applications - ConCache","doc":"Setup project and app dependency in your `mix.exs`:\n\n```elixir\n  ...\n\n  defp deps do\n    [{:con_cache, \"~> 1.0\"}, ...]\n  end\n\n  def application do\n    [applications: [:con_cache, ...], ...]\n  end\n\n  ...\n```\n\nA cache can be started using `ConCache.start` or `ConCache.start_link` functions. Both functions take two arguments - the first one being a list of ConCache options, and the second one a list of GenServer options for the process being started.\n\nTypically you want to start the cache from a supervisor:\n\n```elixir\nSupervisor.start_link(\n  [\n    ...\n    {ConCache, [name: :my_cache, ttl_check_interval: false]}\n    ...\n  ],\n  ...\n)\n```\n\nFor OTP apps, you can generally find this in `lib/ .ex`. In the Phoenix web framework, look in the `start` function and add the worker to the `children` list.\n\nNotice the `name: :my_cache` option. The resulting process will be registered under this alias. Now you can use the cache as follows:\n\n```elixir\n# Note: all of these requests run in the caller process, without going through\n# the started process.\n\nConCache.put(:my_cache, :key, \"value\")         # inserts value or overwrites the old one\nConCache.insert_new(:my_cache, :key, \"value\")  # inserts value or returns {:error, :already_exists}\nConCache.get(:my_cache, :key)\nConCache.delete(:my_cache, :key)\nConCache.size(:my_cache)\n\nConCache.update(:my_cache, :key, fn(old_value) ->\n  # This function is isolated on a row level. Modifications such as update, put, delete,\n  # on this key will wait for this function to finish.\n  # Modifications on other items are not affected.\n  # Reads are always dirty.\n\n  {:ok, \"new_value\"}\nend)\n\n# Similar to update, but executes provided function only if item exists.\n# Otherwise returns {:error, :not_existing}\nConCache.update_existing(:my_cache, :key, fn(old_value) ->\n  {:ok, \"new_value\"}\nend)\n\n# Returns existing value, or calls function and stores the result.\n# If many processes simultaneously invoke this function for the same key, the function will be\n# executed only once, with all others reading the value from cache.\nConCache.get_or_store(:my_cache, :key, fn() ->\n  \"initial_value\"\nend)\n\n# Similar to get_or_store/3 but works with :ok/:error tuples.\n# The value is cached only if the function returns an :ok tuple.\nConCache.fetch_or_store(:my_cache, :key, fn ->\n  case call_api() do\n    # The processed value will be cached and returned as an :ok tuple.\n    {:ok, data} -> {:ok, process_data(data)}\n    # The error tuple is propagated to the caller.\n    {:error, _reason} = error -> error\n  end\nend)\n```\n\nDirty modifiers operate directly on ETS record without trying to acquire the row lock:\n\n```elixir\nConCache.dirty_put(:my_cache, :key, \"value\")\nConCache.dirty_insert_new(:my_cache, :key, \"value\")\nConCache.dirty_delete(:my_cache, :key)\nConCache.dirty_update(:my_cache, :key, fn(old_value) -> ... end)\nConCache.dirty_update_existing(:my_cache, :key, fn(old_value) -> ... end)\nConCache.dirty_get_or_store(:my_cache, :key, fn() -> ... end)\nConCache.dirty_fetch_or_store(:my_cache, :key, fn() -> ... end)\n```","ref":"readme.html#usage-in-otp-applications"},{"type":"extras","title":"Callback - ConCache","doc":"You can register your own function which will be invoked after an element is stored or deleted:\n\n```elixir\n{ConCache, [name: :my_cache, callback: fn(data) -> ... end]}\n\nConCache.put(:my_cache, :key, \"value\")         # fun will be called with {:update, cache_pid, key, value}\nConCache.delete(:my_cache, :key)             # fun will be called with {:delete, cache_pid, key}\n```\n\nThe delete callback is invoked before the item is deleted, so you still have the chance to fetch the value from the cache and do something with it.","ref":"readme.html#callback"},{"type":"extras","title":"TTL - ConCache","doc":"```elixir\n{ConCache, [\n  name: :my_cache,\n  ttl_check_interval: :timer.seconds(1),\n  global_ttl: :timer.seconds(5)\n]}\n```\n\nThis example sets up item expiry check every second, and sets the global expiry for all cache items to 5 seconds. Since ttl_check_interval is 1 second, the item lifetime might be at most 6 seconds.\n\nHowever, the item lifetime is renewed on every modification. Reads don't extend global_ttl, but this can be changed when starting cache:\n\n```elixir\n{ConCache, [\n  name: :my_cache,\n  ttl_check_interval: :timer.seconds(1),\n  global_ttl: :timer.seconds(5),\n  touch_on_read: true\n]}\n```\n\nIn addition, you can manually renew item's ttl:\n\n```elixir\nConCache.touch(:my_cache, :key)\n```\n\nIf you would like to set a custom ttl for specific key, you can pass a `Concache.Item` struct instead of a raw value:\n\n```elixir\nConCache.put(:my_cache, :key, %ConCache.Item{value: \"value\", ttl: :timer.seconds(25)})\n\nConCache.update(:my_cache, :key, fn(old_value) ->\n  {:ok, %ConCache.Item{value: \"new_value\", ttl: :timer.seconds(25)}}\nend)\n```\n\nAnd you can update an item without resetting the item's ttl:\n\n```elixir\nConCache.put(:my_cache, :key, %ConCache.Item{value: \"value\", ttl: :no_update})\n\nConCache.update(:my_cache, :key, fn(old_value) ->\n  {:ok, %ConCache.Item{value: \"new_value\", ttl: :no_update}}\nend)\n```\n\nIf you use ttl value of `:infinity` the item never expires.\n\nTTL check **is not** based on brute force table scan, and should work reasonably fast assuming the check interval is not too small. I broadly recommend `ttl_check_interval` to be at least 1 second, possibly more, depending on the cache size and desired ttl.\n\nIf needed, you may also pass false to `ttl_check_interval`. This effectively stops `con_cache` from checking the ttl of your items:\n\n```elixir\n{ConCache, [\n  name: :my_cache,\n  ttl_check_interval: false\n]}\n```","ref":"readme.html#ttl"},{"type":"extras","title":"Telemetry - ConCache","doc":"As of 1.1.0, ConCache emits [telemetry](https://github.com/beam-telemetry/telemetry) events. This allows the user to instrument their application to collect metrics about cache utilization.\n\nCurrently, ConCache emits the following events:\n\n - `[:con_cache, :stats, :hit]` - when cache key lookup succeeds\n - `[:con_cache, :stats, :miss]` - when cache key is not found\n\nEach event comes with `%ConCache{}` struct within its metadata.\n\nExample handler:\n\n```elixir\ndefmodule MyApp.HitMissRatioTracker do\n  require Logger\n\n  def handle_event([:con_cache, :stats, :hit], _measurements, %{cache: %{name: cache_name}}, _config) do\n    # ... aggregate hits\n  end\n\n  def handle_event([:con_cache, :stats, :miss], _measurements, %{cache: %{name: cache_name}}, _config) do\n    # ... aggregate misses\n  end\nend\n```","ref":"readme.html#telemetry"},{"type":"extras","title":"Supervision - ConCache","doc":"A call to `ConCache.start_link` (or `start`) creates the so called _cache owner process_. This is the process that is the owner of the underlying ETS table and also the process where TTL checks are performed. No other operation (such as get or put) runs in this process.\n\nAs you've seen from the examples above, it's your responsibility to place the cache owner process into your own supervision tree. This gives you the control of cache cleanup when some subtree terminates (since a termination of the owner process will release the ETS table).\n\nIf for some reason `:con_cache` application is terminated, all cache owner processes will be terminated as well, regardless of the fact that they do not reside in the `:con_cache` supervision tree.","ref":"readme.html#supervision"},{"type":"extras","title":"Multiple caches - ConCache","doc":"Sometimes it can be useful to run multiple caches - say, if you need 2 caches with different global expiry values. Even though you can override ttl for each\nitem individually, it might get tedious very quickly.\n\nBy default it's not possible to run multiple caches under the same supervisor because child specification of each cache owner process has `id` equal to `ConCache`.\n\nHowever you can override default child specification and provide unique `id`:\n\n```elixir\ndef start(_type, _args) do\n  Supervisor.start_link(\n    [\n      ...\n      con_cache_child_spec(:my_cache_1, 100),\n      con_cache_child_spec(:my_cache_2, 200)\n      ...\n    ],\n    ...\n  )\nend\n\ndefp con_cache_child_spec(name, global_ttl) do\n  Supervisor.child_spec(\n    {\n      ConCache,\n      [\n        name: name,\n        ttl_check_interval: :timer.seconds(1),\n        global_ttl: :timer.seconds(global_ttl)\n      ]\n    },\n    id: {ConCache, name}\n  )\nend\n```\n\nSee [Supervisor.child_spec/2](https://hexdocs.pm/elixir/Supervisor.html#child_spec/2-examples) for details of this technique.","ref":"readme.html#multiple-caches"},{"type":"extras","title":"Process alias - ConCache","doc":"Functions `ConCache.start` and `ConCache.start_link` return standard `{:ok, pid}` result. You can interface with the cache using this pid. As mentioned, cache operations are not running through this process - the pid is just used to discover the corresponding ETS table.\n\nMost of the time using pid to interface the cache is not appropriate. Just like in examples above, you usually want to give some alias to your cache, and then access it via this alias. In the examples above, we used `name: :some_alias` to provide local alias. Alternatively, you can use following formats for `name` option:\n\n```elixir\n{:global, some_alias}         # globally registered alias\n{:via, module, some_alias}    # registered through some module (e.g. gproc)\n```\n\nIn this case, you can just pass the same tuple to other `ConCache` functions. For example, to use the cache with [gproc](https://github.com/uwiger/gproc), you can do something like this:\n\n```elixir\nConCache.start_link([], name: {:via, :gproc, :my_cache})\n...\nConCache.put({:via, :gproc, :my_cache}, :some_key, :some_value)\n```","ref":"readme.html#process-alias"},{"type":"extras","title":"Testing in your application - ConCache","doc":"Keep in mind that `ConCache` introduces a state to your system. Thus, when you're testing your application, some tests might accidentally compromise the execution of other tests. There are a couple of options to work around that:\n\n1. Use different keys in each test. This could help avoiding tests compromising each other.\n1. Before each test, force restart the `ConCache` process. This will ensure each test runs with the empty cache.\n\n  ```elixir\n  setup do\n    Supervisor.terminate_child(con_cache_supervisor, ConCache)\n    Supervisor.restart_child(con_cache_supervisor, ConCache)\n    :ok\n  end\n  ```\n\n  Where `con_cache_supervisor` is the supervisor from which the `ConCache` process is started.\n\n3. Fetch all keys from the `ets` table, and delete each entry:\n\n  ```elixir\n  setup do\n    :my_cache\n    |> ConCache.ets\n    |> :ets.tab2list\n    |> Enum.each(fn({key, _}) -> ConCache.delete(:my_cache, key) end)\n\n    :ok\n  end\n  ```","ref":"readme.html#testing-in-your-application"},{"type":"extras","title":"Inner workings - ConCache","doc":"","ref":"readme.html#inner-workings"},{"type":"extras","title":"ETS table - ConCache","doc":"The ETS table is always public, and by default it is of _set_ type. Some ETS parameters can be changed:\n\n```elixir\nConCache.start_link(ets_options: [\n  :named_table,\n  {:name, :test_name},\n  :ordered_set,\n  {:read_concurrency, true},\n  {:write_concurrency, true},\n  {:decentralized_counters, true},\n  {:heir, heir_pid}\n])\n```\n\nAdditionally, you can override ConCache, and access ETS directly:\n\n```elixir\n:ets.insert(ConCache.ets(cache), {key, value})\n```\n\nOf course, this completely overrides additional ConCache behavior, such as ttl, row locking and callbacks.\n\n#### Bag and Duplicate Bag\n\nThose types are now supported by ConCache but like ETS, some functions are not supported by those types. Here are the list of functions **not** supported by bag and duplicate bag type tables:\n\n- `update/3`\n- `dirty_update/3`\n- `update_existing/3`\n- `dirty_update_existing/3`\n- `get_or_store/3`\n- `dirty_get_or_store/3`\n- `fetch_or_store/3`\n- `dirty_fetch_or_store/3`","ref":"readme.html#ets-table"},{"type":"extras","title":"Locking - ConCache","doc":"To provide isolation, custom implementation of mutex is developed. This enables that each update operation is executed in the caller process, without the need to send data to another sync process.\n\nWhen a modification operation is called, the ConCache first acquires the lock and then performs the operation. The acquiring is done using the pool of lock processes that reside in the ConCache supervision tree. The pool contains as many processes as there are schedulers.\n\nIf the lock is not acquired in a predefined time (default = 5 seconds, alter with _acquire_lock_timeout_ ConCache parameter) an exception will be generated.\n\nYou can use explicit isolation to perform isolated reads if needed. In addition, you can use your own lock ids to implement bigger granularity:\n\n```elixir\nConCache.isolated(cache, key, fn() ->\n  ConCache.get(cache, key)    # isolated read\nend)\n\n# Operation isolated on an arbitrary id. The id doesn't have to correspond to a cache item.\nConCache.isolated(cache, my_lock_id, fn() ->\n  ...\nend)\n\n# Same as above, but immediately returns {:error, :locked} if lock could not be acquired.\nConCache.try_isolated(cache, my_lock_id, fn() ->\n  ...\nend)\n```\n\nKeep in mind that these calls are isolated, but not transactional (atomic). Once something is modified, it is stored in ETS regardless of whether the remaining calls succeed or fail.\nThe isolation operations can be arbitrarily nested, although I wouldn't recommend this approach.","ref":"readme.html#locking"},{"type":"extras","title":"TTL - ConCache","doc":"When ttl is configured, the owner process works in discrete steps using `:erlang.send_after` to trigger the next step.\n\nWhen an item ttl is set, the owner process receives a message and stores it in its internal structure without doing anything else. Therefore, repeated touching of items is not very expensive.\n\nIn the next discrete step, the owner process first applies the pending ttl set requests to its internal state. Then it checks which items must expire at this step, purges them, and calls `:erlang.send_after` to trigger the next step.\n\nThis approach allows the owner process to do fairly small amount of work in each discrete step.","ref":"readme.html#ttl"},{"type":"extras","title":"Consequences - ConCache","doc":"Due to the locking and ttl algorithms just described, some additional processing will occur in the owner processes. The work is fairly optimized, but I didn't invest too much time in it.\nFor example, lock processes currently use pure functional structures such as `HashDict` and `:gb_trees`. This could probably be replaced with internal ETS table to make it work faster, but I didn't try it.\n\nDue to locking and ttl inner workings, multiple copies of each key exist in memory. Therefore, I recommend avoiding complex keys.","ref":"readme.html#consequences"},{"type":"extras","title":"Status - ConCache","doc":"ConCache has been used in production to manage several thousands of entries served to up to 4000 concurrent clients, on the load of up to 2000 reqs/sec. I don't maintain that project anymore, so I'm not aware of its current status.","ref":"readme.html#status"},{"type":"extras","title":"Copyright and License - ConCache","doc":"Copyright (c) 2013 Saša Jurić\n\nReleased under the MIT License, which can be found in the repository in [`LICENSE`](https://github.com/sasa1977/con_cache/blob/master/LICENSE).","ref":"readme.html#copyright-and-license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}